/*
 * Fujitsu c4110 application panel driver
 * apanelc.c - the userland control program
 *
 * (c) Copyright by Willem van Engen
 * See COPYRIGHT for copyright
 * See CREDITS for credits
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/syscall.h>
#include <string.h>
#include <err.h>
#include "apanel.h"

#define atoix(A) (int)strtol(A,(char**)NULL,0)

int fd;

void apanel_set(int, int, int);
int apanel_get(int, int);
int apanel_get_2(int,int,int);

int main(int argc,char* argv[]) {
  int i,stat;

  if ((fd=open(APANEL_DEV,O_RDWR)) < 0) {
    err(1,APANEL_DEV);
  }

  if (argc==1) goto usage;
  for (i=1;i<argc;i++) {
    /********* LED *********/
    if (strcasecmp(argv[i],"led")==0) {
      if (argc-i<=1) goto ledusage;
      if (strcasecmp(argv[++i],"on")==0) {
	stat=1;
	apanel_set(APANEL_DEV_LED,APANEL_CMD_ONOFF,APANEL_ONOFF_ON);
      } else if(strcasecmp(argv[i],"off")==0) {
        stat=0;
	apanel_set(APANEL_DEV_LED,APANEL_CMD_ONOFF,APANEL_ONOFF_OFF);
      } else if(strcasecmp(argv[i],"blink")==0) {
        if (argc-i<=1) goto blinkusage;
        stat=atoix(argv[++i]);
	if (stat>=apanel_get_2(APANEL_DEV_LED,APANEL_CMD_MIN,APANEL_CMD_BLINK) && 
		stat<=apanel_get_2(APANEL_DEV_LED,APANEL_CMD_MAX,APANEL_CMD_BLINK) ) {
	  apanel_set(APANEL_DEV_LED,APANEL_CMD_BLINK,stat);
        } else {
blinkusage:
          fprintf(stderr,"Blink requires argument from %d to %d.\n",
				  apanel_get_2(APANEL_DEV_LED,APANEL_CMD_MIN,APANEL_CMD_BLINK),
				  apanel_get_2(APANEL_DEV_LED,APANEL_CMD_MAX,APANEL_CMD_BLINK));
	  return 3;
        }
      } else if(strcasecmp(argv[i],"stat")==0 || 
               strcasecmp(argv[i],"status")==0) {
        printf("%d\n",apanel_get(APANEL_DEV_LED,APANEL_CMD_GET));
	  } else if(strcasecmp(argv[i],"duty")==0) {
        if (argc-i<=1) goto dutyusage;
		stat=atoix(argv[++i]);
		if (stat>=apanel_get_2(APANEL_DEV_LED,APANEL_CMD_MIN,APANEL_CMD_DUTY) &&
			stat<=apanel_get_2(APANEL_DEV_LED,APANEL_CMD_MAX,APANEL_CMD_DUTY)) {
			apanel_set(APANEL_DEV_LED,APANEL_CMD_DUTY,stat);
		} else {
dutyusage:
			fprintf(stderr,"Duty requires argument from %d to %d.\n",
				  apanel_get_2(APANEL_DEV_LED,APANEL_CMD_MIN,APANEL_CMD_DUTY),
				  apanel_get_2(APANEL_DEV_LED,APANEL_CMD_MAX,APANEL_CMD_DUTY));
			return 3;
		}
      } else {
ledusage:
        fprintf(stderr,"usage: apanelc led on|off|blink xx|duty xx|stat[us]\n");
	return 3;
      }
    /********* LCD *********/
    } else if(strcasecmp(argv[i],"lcd")==0) {
      if (argc-i<=1) goto lcdusage;
      if (strcasecmp(argv[++i],"stat")==0 ||
		strcasecmp(argv[i],"status")==0) {
	stat=apanel_get(APANEL_DEV_LCD,APANEL_CMD_GET);
	if (stat==0) printf("off\n");
	else if (stat==apanel_get_2(APANEL_DEV_LCD,APANEL_CMD_MAX,
		    			APANEL_CMD_SET)+1)
	    printf("--\n");
	else printf("%d\n",stat);
      } else if (strcasecmp(argv[i],"off")==0) {
	  apanel_set(APANEL_DEV_LCD,APANEL_CMD_ONOFF,APANEL_ONOFF_OFF);
      } else if (strcasecmp(argv[i],"on")==0) {
	  apanel_set(APANEL_DEV_LCD,APANEL_CMD_ONOFF,APANEL_ONOFF_ON);
      } else if (argv[i][0]=='-' && (argv[i][1]=='\0' ||
                  (argv[i][1]=='-' && argv[i][2]=='\0')) ) {
	  apanel_set(APANEL_DEV_LCD,APANEL_CMD_SET,
		  apanel_get_2(APANEL_DEV_LCD,APANEL_CMD_MAX,APANEL_CMD_SET)+1 );
      } else {
        stat=atoix(argv[i]);
        if ( stat>apanel_get_2(APANEL_DEV_LCD,APANEL_CMD_MAX,APANEL_CMD_SET) ||
	  stat<apanel_get_2(APANEL_DEV_LCD,APANEL_CMD_MIN,APANEL_CMD_SET) ) {
lcdusage:
            fprintf(stderr,"usage: apanelc lcd stat[us]|off|-[-]|xxx\n"
                         "  xxx  number from %d to %d to put on lcd\n",
				apanel_get_2(APANEL_DEV_LCD,APANEL_CMD_MIN,APANEL_CMD_SET),
				apanel_get_2(APANEL_DEV_LCD,APANEL_CMD_MAX,APANEL_CMD_SET) );
	    return 3;
        }
	apanel_set(APANEL_DEV_LCD,APANEL_CMD_SET,stat);
      }
    /******** Key commands *********/
    } else if(strcasecmp(argv[i],"btn")==0) {
	printf("cdbutton says   0x%x\n",
			apanel_get(APANEL_DEV_CDBTN,APANEL_CMD_GET) );
	printf("applbutton says 0x%x\n",
			apanel_get(APANEL_DEV_APPBTN,APANEL_CMD_GET) );
    /******** Misc commands ********/
    } else if(strcasecmp(argv[i],"ver")==0) {
	if (ioctl(fd,IOCTL_APANEL_VER,&stat)==-1)
		err(1,"IOCTL_APANEL_VER");
	printf("Driver interface version is 0x%x\n",stat);
    } else if(strcasecmp(argv[i],"dev")==0 || 
		strcasecmp(argv[i],"devices")==0) {
	if (ioctl(fd,IOCTL_APANEL_CAP_GEN,&stat)==-1)
	    err(1,"IOCTL_APANEL_CAP_GEN");
	printf("Detected devices: %s %s %s %s\n",
		stat&APANEL_CAP_GEN_LED==0?"":"LED",
		stat&APANEL_CAP_GEN_LCD==0?"":"LCD",
		stat&APANEL_CAP_GEN_APPBTN==0?"":"APPBTN",
		stat&APANEL_CAP_GEN_CDBTN==0?"":"CDBTN");
    /******** Wrong device *********/
    } else {
usage:
      fprintf(stderr,	"usage: apanelc led on|off|blink xxx|duty xxx\n"
			"       apanelc lcd stat[us]|off|-[-]|xxx\n"
			"       apanelc btn|ver|dev[ices]\n");
      return 3;
    }
  }

  close(fd);
  return 0;
}

void apanel_set(int device, int cmd, int data) {
  struct apanelcmd apcmd={device,cmd,data};
  if (ioctl(fd,IOCTL_APANEL_CMD,&apcmd)==-1)
    err(1,"IOCTL_APANEL_CMD(0x%x,0x%x,0x%x)",device,cmd,data);
}
int apanel_get(int device, int cmd) { return apanel_get_2(device,cmd,0); }
int apanel_get_2(int device, int cmd, int data) {
  struct apanelcmd apcmd={device,cmd,data};

  if (ioctl(fd,IOCTL_APANEL_CMD,&apcmd)==-1)
    err(1,"IOCTL_APANEL_CMD(0x%x,0x%x, )",device,cmd);
  return apcmd.data;
}

