/*
    FJKeyInf client
    
    Copyright (C) 2001 Jochen Eisinger <jochen.eisinger@gmx.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <signal.h>

/* #include <linux/apanel.h> */
#include "apanel.h"

#define VERSION	"fjkeyd - Fujitsu Application Panel Key Daemon v0.1"

#define FJKEYD_PID_FILE	"/var/run/fjkeyd.pid"
#define FJKEYD_DEFAULT_SCRIPTDIR "/etc/apanel"

#define FJKEYD_MODE_DAEMON	0
#define FJKEYD_MODE_RELOAD	1
#define FJKEYD_MODE_STATUS	2
#define FJKEYD_MODE_STOP	3
#define FJKEYD_MODE_HELP	4
#define FJKEYD_MODE_VERSION	5
#define FJKEYD_MODE_INVAL	-1

static int	prog_mode = FJKEYD_MODE_DAEMON;

static int	daemon_pid = -1;

char scriptdir[256]="/etc/apanel";

void parse_options(int argc, char ** args)
{

	if (argc == 1)
		return;

	if (argc > 2) {
		prog_mode = FJKEYD_MODE_INVAL;
		return;
	}

	if (strcmp(args[1],"-r") == 0) {

		prog_mode = FJKEYD_MODE_RELOAD;
		return;

	}

	if (strcmp(args[1],"-s") == 0) {

		prog_mode = FJKEYD_MODE_STATUS;
		return;

	}

	if (strcmp(args[1],"-k") == 0) {

		prog_mode = FJKEYD_MODE_STOP;
		return;
	}

	if (strcmp(args[1],"-h") == 0) {

		prog_mode = FJKEYD_MODE_HELP;
		return;

	}

	if (strcmp(args[1],"-v") == 0) {

		prog_mode = FJKEYD_MODE_VERSION;
		return;

	}

	prog_mode = FJKEYD_MODE_INVAL;
		

}

void get_daemon_pid(void)
{

	FILE	*fp;
	struct	stat buf;
	char	tmp[128];

	if ((fp = fopen(FJKEYD_PID_FILE, "r+")) == NULL)
		return;

	
	fscanf(fp, "%d", &daemon_pid);

	fclose(fp);

	sprintf(tmp, "/proc/%d", daemon_pid);

	if (stat(tmp, &buf) < 0) {

		remove(FJKEYD_PID_FILE);
		daemon_pid = -1;

	}
	
}

void set_daemon_pid(void)
{

	FILE	*fp;

	if ((fp = fopen(FJKEYD_PID_FILE, "w+")) == NULL) {

		fprintf(stderr, "could not write pid-file...\n");
		return;

	}

	fprintf(fp, "%d", daemon_pid);

	fclose(fp);

}

void test_fjkeyinf_dev(void)
{

	int	fdev;

	if ((fdev = open(APANEL_DEV, O_RDONLY)) < 0) {

		fprintf(stderr, "could not find apanel device...\n");
		exit(1);

	}

	close(fdev);

}

void runit(char *name)
{
	char progpath[256];

	/* Run script with sh */
	sprintf(progpath,"%s/%s.sh&",scriptdir,name);
	system(progpath);
}

void poll_fjkeyinf(void)
{

	FILE	*config;
	int	fdev, ctr, i, l;
	int	state;
	char	tmp[256], *p;
	struct	apanelcmd cmd,cmd2;

	fdev = open(APANEL_DEV, O_RDONLY);

	cmd.device = APANEL_DEV_APPBTN;
	cmd.cmd = APANEL_CMD_GET;
	cmd2.device = APANEL_DEV_CDBTN;
	cmd2.cmd = APANEL_CMD_GET;

	for (;;) {

		ioctl(fdev, IOCTL_APANEL_CMD, (unsigned long)&cmd);
		ioctl(fdev, IOCTL_APANEL_CMD, (unsigned long)&cmd2)<<8;
		state = cmd.data + (cmd2.data<<8);

		switch (state) {

			case APANEL_APPBTN_A:
				runit("A");
				sleep(1);
				break;

			case APANEL_APPBTN_B:
				runit("B");
				sleep(1);
				break;

			case APANEL_APPBTN_INTERNET:
				runit("internet");
				sleep(1);
				break;

			case APANEL_APPBTN_EMAIL:
				runit("email");
				sleep(1);
				break;

			case APANEL_CDBTN_STOP<<8:
				runit("stop");
				sleep(1);
				break;

			case APANEL_CDBTN_PLAY<<8:
				runit("play");
				sleep(1);
				break;

			case APANEL_CDBTN_BACK<<8:
				runit("back");
				sleep(1);
				break;

			case APANEL_CDBTN_FORW<<8:
				runit("forw");
				sleep(1);
				break;

			default:
				usleep(500);
		}
		waitpid(-1, NULL, WNOHANG);

	}

	close(fdev);

}

int main(int argc, char ** args)
{

	parse_options(argc, args);

	switch (prog_mode) {

		case FJKEYD_MODE_INVAL:

			fprintf(stderr, "%s: invalid arguments. "
					"Use -h for help.\n", args[0]);
			return 1;

		case FJKEYD_MODE_HELP:

			fprintf(stdout, VERSION "\n");
			fprintf(stdout, "\n%s: [option]\n", args[0]);
			fprintf(stdout, "\n\t-h\tshows this help"
					"\n\t-k\tkills running daemon"
					"\n\t-r\treloads configuration file"
					"\n\t-s\tdisplays status information"
					"\n\t-v\tdisplay version"
					"\n\nWhen invoked without any options "
					"a new daemon is started if possible\n"
				);
			return 0;

		case FJKEYD_MODE_VERSION:
			
			fprintf(stdout, VERSION "\n");
			return 0;

	}

	if (getuid() != 0) {

		fprintf(stderr, "%s: you have to be root\n", args[0]);
		return 1;
	}

	test_fjkeyinf_dev();

	get_daemon_pid();

	switch (prog_mode) {

		case FJKEYD_MODE_STATUS:

			if (daemon_pid == -1) {
				
				fprintf(stdout, "%s: daemon not present\n",
						args[0]);

			} else {
				
				fprintf(stdout, "%s: daemon active (pid: %d)\n",
						args[0], daemon_pid);

			}

			return 0;

		case FJKEYD_MODE_STOP:

			if (daemon_pid == -1) {

				fprintf(stderr, "%s: daemon not present\n",
						args[0]);

				return 1;

			}

			kill(daemon_pid, SIGTERM);
			remove(FJKEYD_PID_FILE);

			return 0;
				
		case FJKEYD_MODE_RELOAD:

			if (daemon_pid != -1) {

				kill(daemon_pid, SIGTERM);
				remove(FJKEYD_PID_FILE);

			}

	}


	switch (daemon_pid = fork()) {


		case -1:

			fprintf(stderr, "%s: fork() failed...\n", args[0]);
			return 1;

		case 0:

			poll_fjkeyinf();
			return 0;

	}

	set_daemon_pid();

	return 0;

}
